#
# $Id: uiClass.pm,v 1.10 2002/03/10 20:36:54 mcooper Exp $
#
# UI Class functions
#

package uiClass;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiFormat;
use uiWidget;
use uiPath;
use uiRetrieve;
use uiDataDisplay;
use uiMeter;
use uiButtons;
use uiIcon;

#
# $Class definetion
#
# Attributes members are as follows:
# 	Table->{$Arrange} - Main hash table containing path data
#			  organized by $Arrange. The key is a pathname.
#	Entries		- Object entries from mcSysInfo
#	Obj		- The object.  i.e. mcDevInfo
#	State		- The state of the entry. i.e. $PS_*
#	Summary		- Text summary describing entry value.
#	Desc		- Text describing what the entry is.
#	Value		- The value of the entry.
#	Widget		- The current display widget
#	ArrangeActive	- Currently active Arrange type
#	ArrangeSelected - The type requested by user - may lag behind Arrange
#	$widname	- Hash pointing to a widget.  One per type (HList...)
#	$WidgetName->{DataDisplayed}
#			- Boolean which indicates the data is current shown
#			  in the widget.
#

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   ClassGet ClassHideWidget ClassShow
		   ClassImage ClassTable
		  );
}

#
# Hide the indicated class widget
#
sub ClassHideWidget {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $FormatName = $Args->{FormatName};
    my $ArrangeName = $Args->{ArrangeName};

    if (!$Class) {
	return;
    }

    if (my $WidgetName = uiWidget::WidgetName({Class=>$Class,
					       ArrangeName=>$ArrangeName,
					       FormatName=>$FormatName})) {
	if (my $Widget = $Class->{$WidgetName}) {
	    Debug("ClassHideWidget: Hiding widget <%s> for class <%s>",
		  $WidgetName, $Class->{Name});
	    $Widget->packForget if Tk::Exists($Widget);
	    $Class->{Widget} = undef;
	}
    }
}

#
# Find Class
#
sub ClassGet {
    my $Name = shift;

    # If $Name is a Path, then take the first member
    if ($Name =~ /$PathSep/) {
	my @p = split($PathSep, $Name);
	$Name = $p[0];
    }

    my @List = @$Classes;
    for (my $i = 0; $i <= $#List; ++$i) {
	if (Eq($Name, $List[$i]->{Name})) {
	    return $List[$i];
	}
    }
}

#
# Return the Table for $Class of type $Arrange.
# If {Set} is provided, set the Table to {Set}.
#
sub ClassTable {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Arrange = $Args->{Arrange};
    my $Set =  $Args->{Set};

    if (!length($Arrange)) {
	$Arrange = uiArrange::ArrangeName({Class=>$Class});
    }
    if (!length($Arrange)) {
	$Arrange = $ArrangeDefault;
    }

##    Debug("ClassTable: Class=<%s> Arrange=<%s> Table=<%s> Set=<%s>",
##	  $Class->{Name}, $Arrange, $Class->{Table}->{$Arrange}, $Set);

    if ($Set) {
	# Set the table to $Set
	$Class->{Table}->{$Arrange} = $Set;
    } else {
	# Return current table
	return $Class->{Table}->{$Arrange};
    }
}

#
# Show Class $Class
#
sub ClassShowClass {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $KeepMeter = $Args->{KeepMeter};
    my $FormatChange = $Args->{FormatChange};
    my $FormatName = FormatName();

    my $NoPath = 0;
    if (!uiPath::PathExists($Class->{Name})) {
	$NoPath = 1;
    }

    my $ChangeState = 0;
    if ($NoPath) {
	if ($Class->{Show} && $Class->{Show} != $Class->{ShowState}) {
	    $ChangeState = 1;
	}
    } else {
	if ($Class->{Show} != $Class->{ShowState}) {
	    $ChangeState = 1;
	}
    }

    Debug("ClassShow: Name=<%s> Show=%d State=%s ChangeState=%d FormatChange=%d NoPath=%s",
	  $Class->{Name}, $Class->{Show}, $Class->{ShowState},
	  $ChangeState, $FormatChange, $NoPath);

    if ($ChangeState) {
	Debug("ClassShow(): Changing <%s> to state %s",
	      $Class->{Name}, $Class->{Show});
	
	if ($Class->{Show}) {
	    Debug("ClassShow(): Setting widget for <%s> to SHOW",
		  $Class->{Name});
	} else {
	    Debug("ClassShow(): Hiding widget <%s> for <%s>",
		  FormatWidget(), $Class->{Name});
	    ClassHideWidget({Class=>$Class, FormatName=>$FormatName});
	}

	$Class->{ShowState} = $Class->{Show};
    }

    if ($Class->{Show}) {
	#
	# Insure that the proper widgets are setup for this class
	#
	if ($ChangeState || $FormatChange) {
	    uiWidget::WidgetSetup({Class=>$Class});
	}
	
	#
	# $NoPath means there is no class data.  If DataDisplayed is FALSE
	# then the class data is present and just needs to be displayed.
	#
	my $WidgetName = WidgetName({Class=>$Class,
				     FormatName=>$FormatName});
	if ($NoPath) {
	    Retrieve({ClassNames=>[ $Class->{Name} ]});
	} elsif (!$Class->{$WidgetName}->{DataDisplayed}) {
	    DataDisplay({Class=>$Class, KeepMeter=>1});
	}

	#
	# Insure that if this was a $ChangeState that the current tab is raised
	#
	if ($ChangeState) {
	    WidgetTabRaise({Tab=>$Class->{Name}});
	}
    }
}

#
# Show Class
#
sub ClassShow {
    my $Args = shift;

    if ($Args->{Class}) {
	$Args->{OneClass} = 1;
	ClassShowClass($Args);
    } else {
	foreach my $Class (@$Classes) {
	    $Args->{Class} = $Class;
	    ClassShowClass($Args);
	}
    }

    WidgetTabSetCurrent();
    ButtonStates();

    if (!$Args->{KeepMeter}) {
	MeterDestroy();
    }
}

#
# Get and set the image for the given class
#
sub ClassImage {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Widget = $Args->{Widget};

    my $image = $Class->{Image . $Widget};

    if (!$image) {
	if (my $icon = GetIcon({Path=>$Class->{Name}})) {
	    #
	    # Store into $Class based on the $Widget name since
	    # the image is only valid on the same $Widget it was created.
	    #
	    $Class->{Image . $Widget} = $image = $Widget->Getimage($icon);
	}
    }

    return $image;
}

1;
